/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 3 of the License.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file gpl-3.0.txt. If not, see
    <http://www.gnu.org/licenses/>.

*/

#include <QtCore/QDebug>
#include <QtXml/QDomDocument>
#include <math.h>

#include "abstracttreeitem.h"
#include "namedparametertreeitem.h"

using namespace NSCore;

class NamedTreeItem : public QTreeItem
{
	public:
		NamedTreeItem( QString n, QTreeItem* p )
			: QTreeItem( n,p )
		{
			//qDebug() << "NamedTreeItem::NamedTreeItem( QString" << n << ", QTreeItem*" << p << ")";
		}

		void init() {
			new NamedTreeItem( "first", this );
			new NamedTreeItem( "second", this );
			if ( dynamic_cast<QTreeItem*>( parent() ) )
				if ( dynamic_cast<QTreeItem*>( parent() )->objectName() == "second" )
					new NamedTreeItem( "third", this );
		}

	private:
		QString debug() const {
			return objectName();
		}
};

int main( int argc, char** argv )
{
	{
		qDebug() << endl << " F I R S T  P a r t: testing QTreeItem" << endl;

		NamedTreeItem* item = new NamedTreeItem( "item", 0 );

		qDebug() << endl << "Empty NamedTreeItem";
		item->debugPrint();

		qDebug() << endl << "Doing item->init(), adding first two items";
		item->init();
		item->debugPrint();

		qDebug() << endl << "Doing init() on item->child( \"first\" ) - should work";
		NamedTreeItem* tmp = dynamic_cast<NamedTreeItem*>( item->child( "first" ) );
		if ( tmp )
			tmp->init();
		else
			qDebug() << " didn't get a valid item!";
		item->debugPrint();

		qDebug() << endl << "Doing init() on item->child( \"first/second\" ) - should work";
		tmp = dynamic_cast<NamedTreeItem*>( item->getNamedChild( "first/second" ) );
		if ( tmp )
			tmp->init();
		else
			qDebug() << " didn't get a valid item!";
		item->debugPrint();

		qDebug() << endl << "Doing init() on item->child( \"second/first\" ) - should NOT work";
		tmp = dynamic_cast<NamedTreeItem*>( item->getNamedChild( "second/first" ) );
		if ( tmp )
			tmp->init();
		else
			qDebug() << " didn't get a valid item!";
		item->debugPrint();

		qDebug() << endl << "Doing init() on item->child( \"first/second/first\" ) - should work";
		tmp = dynamic_cast<NamedTreeItem*>( item->getNamedChild( "first/second/first" ) );
		if ( tmp )
			tmp->init();
		else
			qDebug() << " didn't get a valid item!";
		item->debugPrint();

		qDebug() << endl << "Doing init() on item->child( \"first/second/first/third\" ) - should work";
		tmp = dynamic_cast<NamedTreeItem*>( item->getNamedChild( "first/second/first/third" ) );
		if ( tmp )
			tmp->init();
		else
			qDebug() << " didn't get a valid item!";
		item->debugPrint();

		delete item;
	}

	{
		qDebug() << endl << " S E C O N D  P a r t: testing NamedParameterTreeItem" << endl;

		qDebug() << "Creating empty item";
		NamedParameterTreeItem* item = new NamedParameterTreeItem( "item", QVariant::Invalid, 0 );
		item->debugPrint();

		qDebug() << endl << "adding child \"first\"";
		item->addParameter( "first", QVariant::Invalid );
		item->debugPrint();

		qDebug() << endl << "adding child \"first/first\"";
		item->addParameter( "first/first", 511.0 );
		item->debugPrint();

		qDebug() << endl << "adding child \"first/second\"";
		item->addParameter( "first/second", 1.27 );
		item->debugPrint();

		qDebug() << endl << "adding an empty child of type int";
		item->addParameter( "second", QVariant::Int );
		item->debugPrint();

		qDebug() << endl << "searching for a parameter \"first/first\" and trying to set it to 0.511";
		item->getParameter( "first/first" )->setValue( double( 0.511 ) );
		item->debugPrint();

		qDebug() << endl << "trying to add \"second/first/first\" \"and second/second/first/bla\"";
		item->addParameter( "second/first/first", QVariant::Bool );
		item->debugPrint();
		item->addParameter( "second/second/first/bla", true );
		item->debugPrint();

		delete item;
	}

	{
		qDebug() << endl << " T H I R D  P a r t: testing NamedParameterTreeItem XML" << endl;
		NamedParameterTreeItem* item = new NamedParameterTreeItem( "item", QVariant::Invalid, 0 );
		item->addParameter( "integer", 1 );
		item->addParameter( "double", 2.0 );
		item->addParameter( "string", "bla" );
		item->addParameter( "string/substring", "blub" );
		item->addParameter( "boolean", QVariant( false ) );

		item->debugPrint();

		QString xml = item->toXML();
		qDebug() << xml;

		item->getParameter( "double" )->setValue( M_PI );
		item->getParameter( "boolean" )->setValue( true );
		item->addParameter( "additional after xml", true );
		item->getParameter( "string/substring" )->setValue( QString( "foobar" ) );

		delete item->getParameter( "integer" );
		//delete item->getParameter( "string" );

		item->debugPrint();

		//item->fromXML( xml );
		QDomDocument doc( "testing" );
		doc.setContent( xml );
		NamedParameterTreeItem::setValueFromNode( item, doc.documentElement() );

		item->debugPrint();

		delete item;
	}

	qDebug() << endl << " F I N I S H E D";

	return 0;
}

