/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 3 of the License.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file gpl-3.0.txt. If not, see
    <http://www.gnu.org/licenses/>.

*/

#ifndef DATASETCHAIN_TEST_H
#define DATASETCHAIN_TEST_H

#include "datasetchain.h"

namespace Test {

	class Producer : public NSCore::DataSetProducer
	{
		Q_OBJECT
		public:
			Producer() : NSCore::DataSetProducer(), toProduce( 0 ) {}

			int toProduce;

		public slots:
			void produceData() {
				NSCore::AcqisitionDataSet* data = new NSCore::AcqisitionDataSet( 1, 100 );
				for ( int i=0; i<data->channels(); ++i ) {
					NSCore::ChannelDataSet* channel = data->channel( i );
					for ( int j=0; j<channel->samples(); ++j )
						channel->setSample( j, rand()*2.0/RAND_MAX-1 );
				}
				qDebug() << "Producer::produceData() produced" << data;
				enqueue( data );
				toProduce--;
				if ( toProduce > 0 )
					QTimer::singleShot( 0, this, SLOT( produceData() ) );
				else
					QTimer::singleShot( 1000, QCoreApplication::instance(), SLOT( quit() ) );
			}
			QString toXML() const { return "<bla />"; }
			QString debug() const { return "Producer"; }
	};

	class Consumer : public NSCore::DataSetChainMember
	{
		bool dataArrived( NSCore::AcqisitionDataSet* data ) {
			qDebug() << "Consumer::dataArrived(" << data << ")";
			for ( int i=0; i<data->channels(); ++i ) {
				NSCore::ChannelDataSet* channel = data->channel( i );
				double val=0;
				for ( int j=0; j<channel->samples(); ++j ) {
					val += channel->sample( j );
				}
				val /= channel->samples();
				qDebug() << " Channel" << i << ": Average =" << val;
			}
			//qDebug() << " done.";
			return true;
		}
		public:
			QString toXML() const { return "<bla />"; }
			QString debug() const { return "Consumer"; }

	};

	class ChainMember : public NSCore::DataSetChainMember
	{
		public:
			bool dataArrived( NSCore::AcqisitionDataSet* d ) {
				qDebug() << "ChainMember::dataArrived(" << d << ")";
				return true;
			}
			QString toXML() const { return "<bla />"; }
			QString debug() const { return "ChainMember"; }
	};

};

#endif // DATASETCHAIN_TEST_H
