/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef SIMPLESAFEQUEUE_H
#define SIMPLESAFEQUEUE_H

/**
 * @internal
 */
template<class T> class SimpleSafeQueueElement
{
	public:
		SimpleSafeQueueElement( T& p ) : _p( p ), _next( 0 )
		{
		}

		SimpleSafeQueueElement* next() const { return _next; }

		bool hasNext() { return ( _next==0 )?false:true; }

		T& object() {
			return _p;
		}

		void setNext( SimpleSafeQueueElement* n ) { _next = n; }

	private:
		T& _p;
		SimpleSafeQueueElement* _next;
};

/**
 * @todo docs
 *
 * @todo make it thread-safe
 */
template<class T> class SimpleSafeQueue
{
	public:
		SimpleSafeQueue() : _size( 0 ), _first( 0 ), _last( 0 )
		{
		}

		~SimpleSafeQueue() {
			while ( _size > 0 )
				pop();
		}

		void push( T& n ) {
			SimpleSafeQueueElement<T>* e = new SimpleSafeQueueElement<T>( n );
			if ( _first == 0 )
				_first = e;
			if ( _last != 0 )
				_last->setNext( e );
			_last = e;
			_size++;
		}

		void pop() {
			SimpleSafeQueueElement<T>* e = _first;
			_first = e->next();
			if ( _first == 0 )
				_last = 0;
			delete e;
			_size--;
		}

		T& front() {
			return _first->object();
		}

		int size() const {
			return _size;
		}
	private:
		int _size;
		SimpleSafeQueueElement<T>* _first;
		SimpleSafeQueueElement<T>* _last;
};

#endif // SIMPLESAFEQUEUE_H

