/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#include "diskiobuffers_bin.h"
#include "diskiobuffers_bin.moc"

#include <QtCore/QDebug>
#include <QtCore/QCoreApplication>
#include <QtCore/QBuffer>

namespace NSDiskIO {

	EPOSBinWriter::EPOSBinWriter( QObject* p ) : AbstractWriter( p ), AbstractDiskIO<QDataStream>( QIODevice::WriteOnly ) {
	}
	EPOSBinWriter::~EPOSBinWriter() {
	}

/*	void EPOSBinWriter::customEvent( QEvent* ev ) {
		NSCore::DataSetBufferEvent* bev = dynamic_cast<NSCore::DataSetBufferEvent*>( ev );
		static bool _running = false;
		if ( ev != 0 && bev != 0 && !_running && _stream != 0 ) {
			_running = true;
			while ( !bev->buffer->isEmpty() ) {
				writeDataSet( bev->buffer->first() );
				bev->buffer->deleteFirst();

				// Only proccess events if this is a single-thread app and
				// not doing so would block the app for a long time...
				if ( QThread::currentThread() == QCoreApplication::instance()->thread() )
					QCoreApplication::processEvents();
			}
			_running = false;
		}
	}*/
	void EPOSBinWriter::setFile( QString filename ) {
		AbstractDiskIO<QDataStream>::setFile( filename );
		_file->write( "\%EPOSBIN2\%" );
	}

	void EPOSBinWriter::writeDataSet( NSCore::AcqisitionDataSet& data ) {
		//qDebug() << "EPOSBinWriter::writeDataSet(" << &data << ")";
		if (  _stream != 0 ) {
			//qDebug() << " _stream != 0" << endl << " Writing config";
			*_stream << data.parameter( "samplinginterval" )->toDouble()
			         << data.parameter( "triggerdelay" )->toDouble()
			         << data.parameter( "horizontalposition" )->toDouble()
			         << data.parameter( "timestamp" )->toDouble()
			         << data.channels()
			         << data.channel( 0 )->samples()
			         ;
			for (  int i=0; i<data.channels(); ++i ) {
				//qDebug() << " Writing channel" << i << endl << "  writing config";
				*_stream << data.channel( i )->parameter( "fullscale" )->toDouble()
				         << data.channel( i )->parameter( "offset" )->toDouble()
				         ;
				//qDebug() << "  writing data";
				for (  int j=0; j<data.channel( i )->samples(); ++j ) {
					*_stream << data.channel( i )->sample( j );
				}
			}
		}
		//qDebug() << " ...done";
	}

	EPOSBinReader::EPOSBinReader( QObject* p )
		: AbstractReader( p )
		, AbstractDiskIO<QDataStream>( QIODevice::ReadOnly )
		, _headertoskip( 0 )
	{
	}
	EPOSBinReader::~EPOSBinReader() {
	}
	void EPOSBinReader::reset() {
		if ( _stream ) {
			_stream->device()->reset();
			_file->read( _headertoskip );
		}
	}
	void EPOSBinReader::setFile( QString file ) {
		AbstractDiskIO<QDataStream>::setFile( file );
		_headertoskip = 0;
		if ( _file != 0 && _file->peek( 1 ) == "\%" ) {
			QByteArray tmp = _file->read( 1 );
			QByteArray r;
			do {
				r = _file->read( 1 );
				tmp.append( r );
			} while ( r != "\%" );
			_headertoskip = tmp.size();
			if ( tmp == "\%EPOSBIN2\%" )
				qDebug() << "EPOSBinReader::setFile(" << file << ") seems to be a version 2 file.";
		}
	}
	QString EPOSBinReader::filename() const {
		return AbstractDiskIO<QDataStream>::filename();
	}
	void EPOSBinReader::readSingleDataSet() {
		if ( _stream != 0 ) {

			/// Skip header if necessary
			/*QByteArray tmp = _file->peek( 10 );
			if ( tmp == "\%EPOSBIN2\%" ) {
				qDebug() << "EPOSBinReader::readSingleDataSet() skipping header";
				_file->read( 10 );
			}*/

			double samplinginterval, triggerdelay, horizontalposition, timestamp;
			int channels, samples;
			*_stream >> samplinginterval >> triggerdelay >> horizontalposition >> timestamp >> channels >> samples;

			NSCore::AcqisitionDataSet* data = new NSCore::AcqisitionDataSet( channels, samples );
			data->parameter( "samplinginterval" )->setValue( samplinginterval );
			data->parameter( "triggerdelay" )->setValue( triggerdelay );
			data->parameter( "horizontalposition" )->setValue( horizontalposition );
			data->parameter( "timestamp" )->setValue( timestamp );

			double fullscale, offset, sample;
			for ( int i=0; i<channels; ++i ) {
				NSCore::ChannelDataSet* channel = data->channel( i );
				*_stream >> fullscale >> offset;
				channel->parameter( "fullscale" )->setValue( fullscale );
				channel->parameter( "offset" )->setValue( offset );
				for ( int j=0; j<samples; ++j ) {
					*_stream >> sample;
					channel->setSample( j, sample );
				}
			}

			enqueue( data );
			emit percent( int( pos() * 100.0 / fileSize() ) );
		}
	}



	PragReader::PragReader( QObject* p )
		: AbstractReader( p )
		, AbstractDiskIO<QDataStream>( QIODevice::ReadOnly )
	{
	}
	PragReader::~PragReader() {
	}
	void PragReader::setFile( QString file ) {
		AbstractDiskIO<QDataStream>::setFile( file );
	}
	QString PragReader::filename() const {
		return AbstractDiskIO<QDataStream>::filename();
	}
	void PragReader::readSingleDataSet() {
		if ( _stream != 0 ) {

			QVector<double> tdata;
			qint8 tmp = 0;
			while ( tmp == 0 && !_stream->atEnd() )
				*_stream >> tmp;

			while ( tmp != 0 && !_stream->atEnd() ) {
				tdata.push_back( tmp );
				*_stream >> tmp;
			}

			NSCore::AcqisitionDataSet* data = new NSCore::AcqisitionDataSet( 2/*channels*/, ( tdata.size() - 1 )/2 /*samples*/ );

			for ( int j=0; j<2; ++j ) {
				NSCore::ChannelDataSet* channel = data->channel( j );
				for ( int i=0; i<channel->samples(); ++i ) {
					channel->setSample( i, ( tdata[ i*2 + j ]-127 )/258 );
				}
				channel->parameter( "fullscale" )->setValue( 1 );
			}

			enqueue( data );
			emit percent( int( pos() * 100.0 / fileSize() ) );
		}
	}

	EPOSStrippedBinWriter::EPOSStrippedBinWriter( QObject* p ) : AbstractWriter( p ), AbstractDiskIO<QDataStream>( QIODevice::WriteOnly ) {
	}
	EPOSStrippedBinWriter::~EPOSStrippedBinWriter() {
	}
	void EPOSStrippedBinWriter::setFile( QString filename ) {
		AbstractDiskIO<QDataStream>::setFile( filename );
		_file->write( "\%EPOSStrippedBin\%" );
	}
	void EPOSStrippedBinWriter::writeDataSet( NSCore::AcqisitionDataSet& data ) {
		if ( _stream == 0 )
			return;

		*_stream
		    << data.parameter( "samplinginterval" )->toDouble()
		    << data.parameter( "triggerdelay" )->toDouble()
		    << data.parameter( "horizontalposition" )->toDouble()
		    << data.parameter( "timestamp" )->toDouble()
		    << data.channels();
		for ( int i=0; i<data.channels(); ++i ) {
			resultsToStream( data.channel( i )->results(), *_stream );
		}
	}
	void EPOSStrippedBinWriter::resultsToStream( NSCore::NamedQVariantTreeItem* tree, QDataStream& stream ) {
		QVariant* tmp = tree;
		stream << tree->name() << *tmp << tree->childs();
		foreach( QString n, tree->childlist() ) {
			resultsToStream( tree->child<NSCore::NamedQVariantTreeItem*>( n ), stream );
		}
	}


	EPOSStrippedBinReader::EPOSStrippedBinReader( QObject* p )
		: AbstractReader( p )
		, AbstractDiskIO<QDataStream>( QIODevice::ReadOnly )
		, _headertoskip( 0 )
	{
	}
	EPOSStrippedBinReader::~EPOSStrippedBinReader() {
	}
	void EPOSStrippedBinReader::reset() {
		if ( _stream ) {
			_stream->device()->reset();
			_file->read( _headertoskip );
		}
	}
	void EPOSStrippedBinReader::setFile( QString file ) {
		AbstractDiskIO<QDataStream>::setFile( file );
		_headertoskip = 0;
		if ( _file != 0 && _file->peek( 1 ) == "\%" ) {
			QByteArray tmp = _file->read( 1 );
			QByteArray r;
			do {
				r = _file->read( 1 );
				tmp.append( r );
			} while ( r != "\%" );
			_headertoskip = tmp.size();
			if ( tmp == "\%EPOSStrippedBin\%" )
				qDebug() << " Version one file";
		}
	}
	QString EPOSStrippedBinReader::filename() const {
		return AbstractDiskIO<QDataStream>::filename();
	}
	void EPOSStrippedBinReader::readSingleDataSet() {
		if ( _stream != 0 ) {
			/*QByteArray tmp = _file->peek( 17 );
			if ( tmp == "\%EPOSStrippedBin\%" )
				_file->read( 17 );*/
			int channels;
			double samplinginterval, triggerdelay, horizontalposition, timestamp;
			*_stream >> samplinginterval >> triggerdelay >> horizontalposition >> timestamp >> channels;

			NSCore::AcqisitionDataSet* data = new NSCore::AcqisitionDataSet( channels, 0 );
			data->parameter( "samplinginterval" )->setValue( samplinginterval );
			data->parameter( "triggerdelay" )->setValue( triggerdelay );
			data->parameter( "horizontalposition" )->setValue( horizontalposition );
			data->parameter( "timestamp" )->setValue( timestamp );

			for ( int i=0; i<channels; ++i )
				resultsFromStream( data->channel( i )->results(), *_stream, data->channel( i ) );

			enqueue( data );
			emit percent( int( pos() * 100.0 / fileSize() ) );
		}
	}
	void EPOSStrippedBinReader::resultsFromStream( NSCore::NamedQVariantTreeItem* tree, QDataStream& stream, NSCore::ChannelDataSet* ch ) {
		QString name;
		stream >> name;
		QVariant value( stream );
		int childs;
		stream >> childs;
		if ( name.startsWith( "generation " ) && name.section( " ", -1 ).toInt() > 0 ) {
			ch->newGeneration();
			ch->popGeneration();
		}
		NSCore::NamedQVariantTreeItem* tmp = 0;
		if ( name == tree->name() )
			tmp = tree;
		else
			tmp = tree->child<NSCore::NamedQVariantTreeItem*>( name );
		if ( tmp )
			tmp->setValue( value );
		else {
			tree->addQVariant( name, value );
			tmp = tree->child<NSCore::NamedQVariantTreeItem*>( name );
			//qWarning() << " child" << name << " does not seem to exist. created it.";
		}
		for ( int i=0; i<childs; ++i )
			resultsFromStream( tmp, stream, ch );
	}

};

