/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#include "evaluationpath_view.h"
#include "evaluationpath_view.moc"

#include <QtGui/QPainter>
#include <QtGui/QLabel>
#include <QtCore/QDebug>
#include <QtGui/QPushButton>
#include <QtGui/QMessageBox>

#include <parameter_gui.h>
#include <listdialog.h>
#include <plugins_interface.h>

EvaluationPathItem_View::EvaluationPathItem_View( NSEvaluation::PluginItem* item, QWidget* p )
	: QFrame( p )
	, _item( item )
{
	QVBoxLayout* _layout = new QVBoxLayout( this );
	_layout->setMargin( 2 );
	_layout->setSpacing( 0 );
	setFrameStyle( QFrame::Plain|QFrame::Panel );
	setLineWidth( 1 );

	QHBoxLayout* _layout2 = new QHBoxLayout( 0 );
	_layout2->setMargin( 0 );
	_layout2->setSpacing( 0 );
	QLabel* label = new QLabel( QString( "<qt><b>%1</b>: %2</qt>" ).arg( _item->type() ).arg( _item->name() ), this );
	label->setToolTip( _item->help() );
	label->setWordWrap( true );
	_layout2->addWidget( label, 200 );

	QPushButton* btnrm = new QPushButton( "rm", this );
	connect( btnrm, SIGNAL( clicked() ), this, SLOT( deletePlugin() ) );
	_layout2->addWidget( btnrm, 1 );

	_layout->addLayout( _layout2, 1 );
	_layout->addSpacing( 3 );
	if ( *( _item->config() ) != 0 )
		_layout->addWidget( new ParameterGui( *( _item->config() ), this ), 200 );
}
EvaluationPathItem_View::~EvaluationPathItem_View() {
}

void EvaluationPathItem_View::deletePlugin() {
	//qDebug() << "EvaluationPathItem_View::deletePlugin()";
	if (
		QMessageBox::question( this,
			"Delete Plugin?",
			"You are about to delete a plugin from your evaluation-chain. Are you really sure you want to do this?",
			QMessageBox::Yes, QMessageBox::No|QMessageBox::Default
		) == QMessageBox::Yes
	)
		emit deletePlugin( _item );
}


EvaluationPath_ViewItem::EvaluationPath_ViewItem( /*NSEvaluation::PluginItem*/ NSCore::QTreeItem* item, QWidget* p )
	: QFrame( p )
	, _item( item, this )
	, _layout1( new QVBoxLayout( this ) )
	, _layout2( new QHBoxLayout( 0 ) )
{
	_view = new EvaluationPathItem_View( item, this );

	_layout1->setMargin( 0 );
	_layout1->setSpacing( 0 );
	_layout2->setMargin( 0 );
	_layout2->setSpacing( 0 );
	_layout1->addWidget( _view, 1 );
	_layout1->addLayout( _layout2, 200 );

	connect( _view, SIGNAL( deletePlugin( NSEvaluation::PluginItem* ) ),
	         this,  SIGNAL( deletePlugin( NSEvaluation::PluginItem* ) ) );

	connect( &_item, SIGNAL( deleted() ), this, SLOT( pluginDeleted() ) );

	if ( item->type() != "AnalyzerItem" ) {
		QPushButton* btnnew = new QPushButton( "*", this );
		_layout2->addWidget( btnnew, 1, Qt::AlignTop|Qt::AlignCenter );
		connect( btnnew, SIGNAL( clicked() ), this, SLOT( newPlugin() ) );
	}
}
EvaluationPath_ViewItem::~EvaluationPath_ViewItem() {
}

void EvaluationPath_ViewItem::newPlugin() {
	QStringList plugins = NSEvaluation::PluginFactory::the()->plugins();
	ListDialog* dialog = new ListDialog( "Choose Plugin", "Here you can select the plugin you want to insert.", plugins, this );

	if ( dialog->exec() == 1 )
		emit newPlugin( plugins[ dialog->listWidget->currentRow() ], *_item );

	delete dialog;
}

void EvaluationPath_ViewItem::pluginDeleted() {
	//qDebug() << "EvaluationPath_ViewItem::pluginDeleted()";
	deleteLater();
}

void EvaluationPath_ViewItem::newPlugin( NSEvaluation::PluginItem* plugin, NSEvaluation::PluginItem* parent ) {
	//qDebug() << "EvaluationPath_ViewItem::newPlugin(" << plugin << "," << parent << ")";
	if ( parent == *_item ) {
		_childs.push_back( new EvaluationPath_ViewItem( plugin, this ) );
		_layout2->insertWidget( _layout2->count()-1, _childs.last(), 100, Qt::AlignTop );

		connect( _childs.last(), SIGNAL( newPlugin( QString, NSEvaluation::PluginItem* ) ),
		         this,           SIGNAL( newPlugin( QString, NSEvaluation::PluginItem* ) ) );
		connect( _childs.last(), SIGNAL( deletePlugin( NSEvaluation::PluginItem* ) ),
		         this,           SIGNAL( deletePlugin( NSEvaluation::PluginItem* ) ) );

		_childs.last()->show();

	} else
		foreach( EvaluationPath_ViewItem* child, _childs )
			if ( child != 0 )
				child->newPlugin( plugin, parent );
}

EvaluationPath_View::EvaluationPath_View( QWidget* p )
	: QFrame( p )
	, _path( 0 )
	, _layout( new QVBoxLayout( this ) )
	, _view( 0 )
{
	//setFrameStyle( QFrame::StyledPanel|QFrame::Sunken );
	_layout->setMargin( 2 );
}
EvaluationPath_View::~EvaluationPath_View() {
}

void EvaluationPath_View::setPath( NSEvaluation::EvaluationPath* p ) {
	_path = p;
	if ( _path ) {
		connect( _path, SIGNAL( pluginsChanged() ),
		         this,  SLOT( pathChanged() ) );
		pathChanged();
		connect( _path, SIGNAL( newPlugin( NSEvaluation::PluginItem*, NSEvaluation::PluginItem* ) ),
		         this,  SLOT( newPlugin( NSEvaluation::PluginItem*, NSEvaluation::PluginItem* ) ) );
	}
}

void EvaluationPath_View::pathChanged() {
	//EvaluationPath_ViewItem * child = findChild<EvaluationPath_ViewItem *>( "" );
	while ( _view != 0 ) {
		//qDebug() << " Found " << child << " as a child that is a EvaluationPath_ViewItem*";
		delete _view;
		//child = findChild<EvaluationPath_ViewItem *>( "" );
		_view = 0;
	}

	if ( _path != 0 ) {
		_view = new EvaluationPath_ViewItem( _path, this );
		_layout->addWidget( _view );
		_view->show();
		connect( _view, SIGNAL( newPlugin( QString, NSEvaluation::PluginItem* ) ), this, SLOT( newPlugin( QString, NSEvaluation::PluginItem* ) ) );
		// This doesn't work because its a connection across threads with a custom type...
		//connect( _view, SIGNAL( newPlugin( QString, NSEvaluation::PluginItem* ) ), _path, SLOT( newPlugin( QString, NSEvaluation::PluginItem* ) ) );
		connect( _view, SIGNAL( deletePlugin( NSEvaluation::PluginItem* ) ), this, SLOT( deletePlugin( NSEvaluation::PluginItem* ) ) );
	}
}

void EvaluationPath_View::newPlugin( QString type, NSEvaluation::PluginItem* parent ) {
	//qDebug() << "EvaluationPath_View::newPlugin(" << type << ", " << parent << ")";
	_path->newPlugin( type, parent );
}
void EvaluationPath_View::deletePlugin( NSEvaluation::PluginItem* plugin ) {
	//qDebug() << "EvaluationPath_View::deletePlugin(" << plugin << ")";
	_path->deletePlugin( plugin );
}
void EvaluationPath_View::newPlugin( NSEvaluation::PluginItem* plugin, NSEvaluation::PluginItem* parent ) {
	qDebug() << "EvaluationPath_View::newPlugin(" << plugin << "," << parent << ")";
	if ( _view )
		_view->newPlugin( plugin, parent );
}

