/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#include "filterexpression.h"

#include <QtCore/QDebug>

namespace NSEvaluation {

FilterExpression::FilterExpression( NSCore::QTreeItem* p ) : NSCore::MathExpression( p ), filter( false ) {
	setObjectName( "Filter" );

	startTimer( 500 );
}

double FilterExpression::value( NSCore::AcqisitionDataSet& data ) {
	if ( !filter )
		return 1;

	double ret = 0;
	foreach( NSCore::MathExpression* item, childobjects<NSCore::MathExpression>() ) {
		ret += item->value( data );
	}
	return ret;
}

void FilterExpression::guiSetValue( QVariant n ) {
	if ( n.canConvert( QVariant::Bool ) )
		filter = n.toBool();
}

Qt::ItemFlags FilterExpression::guiFlags() const {
	return Qt::ItemIsEditable | NSCore::MathExpression::guiFlags();
}

void FilterExpression::guiExecAction( QString action, NSCore::GuiServer_Interface* gui ) {
	//qDebug() << "FilterExpression::guiExecAction(" << action << "," << gui << ")";
	if ( action == "Add Expression" ) {
		addChild( this, gui );
	} else
		MathExpression::guiExecAction( action, gui );
}

void FilterExpression::addChild( NSCore::MathExpression* pexp, NSCore::GuiServer_Interface* gui ) {
	//qDebug() << "FilterExpression::addChild(" << pexp << "," << gui << ")";
	QString type = gui->getItem( "New MathExpression to add", "Add", NSCore::MathExpressionFromXMLCreator::the()->availableElements() );
	if ( !type.isEmpty() ) {
		_childsToAdd.push_back( QPair<QString,NSCore::MathExpression*>( type, pexp ) );
	}
}

void FilterExpression::deleteMe( NSCore::MathExpression* exp, NSCore::GuiServer_Interface* gui ) {
	//qDebug() << "FilterExpression::deleteMe(" << exp << "," << gui << ")";
	if ( gui->messageYesNo( "Really?", "Do you really want to remove this Expression?" ) ) {
		_childsToDelete.append( exp );
	}
}

void FilterExpression::workOnCache() {
	while ( !_childsToAdd.isEmpty() ) {
		QPair<QString,NSCore::MathExpression*> p = _childsToAdd.takeFirst();
		NSCore::MathExpressionFromXMLCreator::the()->fromXML( QString( "<%1 />" ).arg( p.first ), p.second );
	}
	while ( !_childsToDelete.isEmpty() )
		delete _childsToDelete.takeFirst();
}

void FilterExpression::timerEvent( QTimerEvent* ev ) {
	workOnCache();
}

};

