/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#include "plugins_interface.h"

#include <math.h>

namespace NSEvaluation {

	Plugin::Plugin( NSCore::QTreeItem* p )
		: _config( 0 )
	{
		NSCore::NamedParameterTreeItem* tmp = dynamic_cast<NSCore::NamedParameterTreeItem*>( p );
		if ( tmp )
			_config = NSCore::NamedParameterTreeItemPointer( tmp );
		else
			_config = NSCore::NamedParameterTreeItemPointer( new NSCore::NamedParameterTreeItem( "options", QVariant::Invalid, p ) );
	}
	NSCore::NamedParameterTreeItemPointer Plugin::config() {
		return NSCore::NamedParameterTreeItemPointer( _config );
	}


	PluginFactory_Client::PluginFactory_Client() {
		//qDebug() << "PluginFactory_Client::PluginFactory_Client()";
		PluginFactory::the()->registerClient( this );
	}


	PluginFactory* PluginFactory::the() {
		static PluginFactory* factory= new PluginFactory();
		return factory;
	}

	void PluginFactory::registerClient( PluginFactory_Client* client ) {
		//qDebug() << "PluginFactory::registerClient(" << client << ") this =" << this;
		_factories.push_back( client );
	}

	Plugin* PluginFactory::newPlugin( QString s, NSCore::QTreeItem* p ) {
		//qDebug() << "PluginFactory::newPlugin(" << s << ") this =" << this;
		for ( QList<PluginFactory_Client*>::Iterator it = _factories.begin(); it != _factories.end(); ++it ) {
			if ( ( *it )->canCreate().contains( s ) )
				return ( *it )->create( s, p );
		}
		return 0;
	}

	QStringList PluginFactory::plugins() const {
		QStringList tmp;
		foreach( PluginFactory_Client* client, _factories ) {
			tmp += client->canCreate();
		}
		return tmp;
	}

	QString PluginFactory::plugintype( QString plugin ) const {
		foreach( PluginFactory_Client* client, _factories ) {
			if ( client->canCreate().contains( plugin ) )
				return client->type( plugin );
		}
		return "";
	}

};

