/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef CHARTVIEW_H
#define CHARTVIEW_H

#include <QtGui/QAbstractItemView>

class ChartView : public QAbstractItemView
{
	Q_OBJECT
	Q_PROPERTY( bool FWHMLine READ FWHMLine WRITE setFWHMLine )
	Q_PROPERTY( bool LogarithmicScale READ logarithmicScale WRITE setLogarithmicScale )
	public:
		ChartView( QWidget* =0 );
		~ChartView();

		QRect visualRect( const QModelIndex& ) const;
		void scrollTo( const QModelIndex&, ScrollHint =EnsureVisible );
		QModelIndex indexAt( const QPoint& ) const;

		bool logarithmicScale() const { return _logscale; }
		bool FWHMLine() const { return _fwhmline; }

	public slots:
		void setLogarithmicScale( bool n ) { _logscale = n; update(); viewport()->update(); }
		void setFWHMLine( bool n ) { _fwhmline = n; viewport()->update(); }

	protected slots:
		void dataChanged( const QModelIndex&, const QModelIndex& );
		void rowsInserted( const QModelIndex&, int, int );
		void rowsAboutToBeRemoved( const QModelIndex&, int, int );

	protected:
		bool edit( const QModelIndex&, EditTrigger, QEvent* );
		QModelIndex moveCursor( QAbstractItemView::CursorAction, Qt::KeyboardModifiers );

		int horizontalOffset() const;
		int verticalOffset() const;

		bool isIndexHidden( const QModelIndex& ) const;

		void setSelection( const QRect&, QItemSelectionModel::SelectionFlags );

		void mouseReleaseEvent( QMouseEvent* );
		void paintEvent( QPaintEvent* );
		void resizeEvent( QResizeEvent* );
		void scrollContentsBy( int, int );

		QRegion visualRegionForSelection( const QItemSelection& ) const;

		void contextMenuEvent( QContextMenuEvent* );

	private:
		QString _title;
		int validItems;
		double xmin, xmax;
		double ymin, ymax;
		bool _dirty;
		bool _logscale;
		bool _fwhmline;
};

#endif // CHARTVIEW_H

