/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 3 of the License.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file gpl-3.0.txt. If not, see
    <http://www.gnu.org/licenses/>.

*/

#include "osapwindow.h"
#include "osapwindow.moc"

#include <acqiristabs_v2_widget.h>
#include <osciwidget_v3.h>
#include <QtCore/QFile>
#include <QtCore/QDataStream>
#include <QtCore/QDebug>
#include <QtCore/QLibrary>
#include <QtGui/QFileDialog>
#include <QtGui/QVBoxLayout>
#include <QtGui/QMenuBar>
#include <QtGui/QScrollArea>
#include <QtGui/QTreeView>

#include <spectrummodel.h>
#include <spectrumgui.h>
#include <resulttospectrum.h>
#include <stackediconwidget.h>
#include <qtreeitemmodel.h>
#include <qtreeitemmodeldelegate.h>
#include <graphicalguiserver.h>

DigitizerWidget::DigitizerWidget( NSAcqiris::Digitizer* d, NSEvaluation::EvaluationPath* path, ResultsToSpectrum* res2spec, QWidget* p )
	: QWidget( p )
	, _acqiristabs( new AcqirisTabsV2Widget( d, 0 ) )
	, _stacked( new StackedIconWidget( this ) )
	, _results2spectrum( res2spec )
	//, _pathview( 0 /*new EvaluationPath_View( 0 )*/ )
	, _path( path )
	, _digitizer( d )
	, _layout( new QVBoxLayout( this ) )
{
	qDebug() << "DigitizerWidget::DigitizerWidget()";
	_layout->setSpacing( 2 );
	_layout->setMargin( 0 );

	QMenuBar* menubar = new QMenuBar( this );
	_layout->setMenuBar( menubar );

	QMenu* filemenu = menubar->addMenu( "File" );
	filemenu->addAction( "Load XML-File...", this, SLOT( openXMLFile() ) );
	filemenu->addAction( "Save to XML-File...", this, SLOT( saveXMLFile() ) );

	_layout->addWidget( _stacked );

	_stacked->addWidget( "Digitizer", _acqiristabs, ":/icons/cr48-action-load_pulses.png" );

	typedef void ( *Myprototype )();
	Myprototype myfunc = ( Myprototype ) QLibrary::resolve( "libplugins", "start" );
	if ( myfunc )
		myfunc();

	/*QScrollArea* scrollarea = new QScrollArea( 0 );
	_pathview = new EvaluationPath_View( scrollarea );
	scrollarea->setWidget( _pathview );
	scrollarea->setWidgetResizable( true );
	_stacked->addWidget( "Evaluation Path", scrollarea, ":/icons/cr48-actions-evaluate_pulses.png" );
	_pathview->setPath( _path );
	_pathview->show();*/

	QTreeItemModelDelegate* delegate = new QTreeItemModelDelegate( new GraphicalGuiServer( this ), 0 );

	NSCore::QTreeItemModel* evalmodel = new NSCore::QTreeItemModel( this );
	evalmodel->setRootItem( _path );
	QTreeView* evalview = new QTreeView( 0 );
	evalview->setModel( evalmodel );
	evalview->setItemDelegate( delegate );
	evalview->resizeColumnToContents( 0 );
	_stacked->addWidget( "Evaluation Path", evalview, ":/icons/cr48-action-evaluate_pulses.png" );

	connect( _results2spectrum, SIGNAL( numberOfSpectraChanged() ), this, SLOT( checkSpectra() ) );

	NSCore::QTreeItemModel* res_model = new NSCore::QTreeItemModel( this );
	res_model->setRootItem( _results2spectrum );
	QTreeView* res_gui = new QTreeView( 0 );
	res_gui->setItemDelegate( delegate );
	res_gui->setModel( res_model );
	res_gui->resizeColumnToContents( 0 );
	_stacked->addWidget( "Res2Spec", res_gui, ":icons/kde/cr48-apps-edu_mathematics.png" );

	OsciWidgetV3* osci = new OsciWidgetV3( 0 );
	_stacked->addWidget( "Oscilloscop", osci, ":/icons/cr48-action-osci.png" );

	_path->registerAtProducer( d );
	_results2spectrum->registerAtProducer( _path );
	osci->chainMember()->registerAtProducer( _results2spectrum );

	qDebug() << "DigitizerWidget::DigitizerWidget() finished";
}
DigitizerWidget::~DigitizerWidget() {
}

void DigitizerWidget::openXMLFile() {
	QString path = QFileDialog::getOpenFileName(  this, 0, 0, "XML-Files (*.xml)\nPathXML-Files (*path.xml)" );
	qDebug() << "Will try to open" << path << "for reading";

	if (  QFile::exists(  path ) ) {
		QFile file(  path );
		if (  file.open(  QIODevice::ReadOnly | QIODevice::Text ) ) {
			QTextStream stream(  &file );
			openXML(  stream.readAll() );
			file.close();
		}
	}
}
void DigitizerWidget::openXML(  QString xml ) {
	QDomDocument doc(  "eode" );
	doc.setContent(  xml );

	QDomNodeList list;

	qDebug() << "Reading <digitizers />'s from XML";
	list = doc.elementsByTagName( "digitizer" );
	for ( int i=0; i<list.size(); ++i ) {
		qDebug() << i << list.at( i ).nodeName();
		if ( ! list.at( i ).attributes().namedItem( "name" ).isNull() && ( list.at( i ).attributes().namedItem( "name" ).nodeValue() == _digitizer->objectName() ) )
			NSCore::NamedParameterTreeItem::setValueFromNode( *( _digitizer->config() ), list.at( i ).firstChild() );
	}

	qDebug() << "Reading <evaluationpath />'s from XML";
	list = doc.elementsByTagName(  "evaluationpath" );
	for (  int i=0; i<list.size(); ++i ) {
		qDebug() << i << list.at(  i ).nodeName();
		_path->parseXML(  list.at(  i ) );
	}

	//qDebug() << "Deleting all result2spectrum-paths";
	//while (  _results2spectrum->paths() > 0 )
	//	_results2spectrum->removeAllPaths();
	qDebug() << "Reading all <resultstospectrum />'s from XML";
	list = doc.elementsByTagName(  "resultstospectrum" );
	for (  int i=0; i<list.size(); ++i ) {
		qDebug() << i << list.at(  i ).nodeName();
		_results2spectrum->addPath(  list.at(  i ) );
	}
}
void DigitizerWidget::saveXMLFile() {
	QString path = QFileDialog::getSaveFileName( this, 0, 0, "XML-Files (*.xml)" );
	if ( path.isEmpty() )
		return;

	QFile file( path );
	if ( file.open( QIODevice::WriteOnly | QIODevice::Text ) ) {
		QTextStream stream( &file );
		stream << QString( "<eode>\n<digitizer name=\"%1\">%2</digitizer>\n%3\n%4\n</eode>\n" ).arg( _digitizer->objectName() ).arg( _digitizer->config()->toXML() ).arg( _path->toXML() ).arg( _results2spectrum->toXML() );
		file.close();
	}
}

void DigitizerWidget::checkSpectra() {
	qDebug() << "EodeMainWindow::checkSpectra()";
	QStringList officialspectra = _results2spectrum->spectra();
	qDebug() << officialspectra;
	foreach(  QString item, _spectrumguis.keys() ) {
		//qDebug() << "Checking wether" << item << " is a gui without a model";
		if (  ! officialspectra.contains(  item ) ) {
			//qDebug() << " Apparently not :-(";
			SpectrumGui* tmp = _spectrumguis.take(  item );
			delete tmp;
		}
	}
	foreach(  QString item, officialspectra ) {
		//qDebug() << "Checking wether" << item << "has a gui.";
		if (  ! _spectrumguis.keys().contains(  item ) ) {
			//qDebug() << " Apparently not, will create a gui";
			SpectrumGui* tmp = new SpectrumGui(  0 );
			tmp->setModel(  _results2spectrum->spectrum(  item ) );
			_spectrumguis[  item ] = tmp;
			tmp->show();
			_stacked->addWidget(  item, tmp, ":/icons/cr48-actions-spectrum.png" );
		}
	}
}

OSAPWindow::OSAPWindow( QStringList arguments, QWidget* p )
	: QTabWidget( p )
	, _acqiris( 0 )
{
	qDebug() << "OSAPWindow::OSAPWindow( QStringList" << arguments << ", QWidget*" << p << ")";
	NSCore::Thread_Template<NSAcqiris::Acqiris> *acqiristhread = new NSCore::Thread_Template<NSAcqiris::Acqiris>( this );
	acqiristhread->start( QThread::HighPriority );
	while ( acqiristhread->item() == 0 )
		usleep( 100 );
	_threads.push_back( acqiristhread );

	init( acqiristhread->item() );

	QStringList files;
	foreach( QString arg, arguments )
		if ( QFile::exists( arg ) )
			files.append( arg );

	qDebug() << " files on the commandline" << files.join( "," );

	for ( int i=0; i<files.size() && i<_digiwidgets.size(); ++i ) {
		QFile f( files[ i ] );
		if ( f.open( QIODevice::ReadOnly ) ) {
			QString xml = f.readAll();
			_digiwidgets[ i ]->openXML( xml );
		}
	}

	if ( arguments.contains( "--autostart" ) )
		for ( int i=0; i<_acqiris->digitizers(); ++i )
			_acqiris->digitizer( i )->startAcqisition();

	qDebug() << "OSAPWindow::OSAPWindow() finished";
}

OSAPWindow::OSAPWindow( NSAcqiris::Acqiris* acqiris, QWidget* p )
	: QTabWidget( p )
	, _acqiris( 0 )
{
	qDebug() << "OSAPWindow::OSAPWindow( NSAcqiris::Acqiris*" << acqiris << ", QWidget*" << p << ")";
	init( acqiris );
	qDebug() << "OSAPWindow::OSAPWindow() finished";
}
OSAPWindow::~OSAPWindow() {
	qDebug() << "OSAPWindow::~OSAPWindow()";
	while ( ! _threads.isEmpty() ) {
		qDebug() << "Stopping thread: " << _threads.first();
		_threads.first()->quit();
		while ( _threads.first()->isRunning() )
			usleep( 100 );
		delete _threads.takeFirst();
	}
	qDebug() << "OSAPWindow::~OSAPWindow() finished.";
}

void OSAPWindow::init( NSAcqiris::Acqiris* acqiris ) {
	_acqiris = acqiris;
	{
		QScrollArea* tmp = new QScrollArea( this );
		tmp->setWidgetResizable( true );
		addTab( tmp, "TV" );
		NSCore::QTreeItemModel* model = new NSCore::QTreeItemModel( this );
		model->setRootItem( acqiris );
		QTreeView* tv = new QTreeView( 0 );
		tv->setModel( model );
		tmp->setWidget( tv );
	}
	for ( int i=0; i<_acqiris->digitizers(); ++i ) {
		NSEvaluation::EvaluationThread* thread = new NSEvaluation::EvaluationThread( this );
		NSCore::Thread_Template<ResultsToSpectrum>* thread2 = new NSCore::Thread_Template<ResultsToSpectrum>( this );
		thread->start( QThread::LowPriority );
		thread2->start();
		while ( thread->item() == 0 || thread2->item() == 0 ) {
			usleep( 100 );
			qDebug() << " waiting for thread->path()";
		}
		DigitizerWidget* tmp = new DigitizerWidget( _acqiris->digitizer( i ), thread->item(), thread2->item(), 0 );
		_digiwidgets.push_back( tmp );
		_threads.push_back( thread );
		_threads.push_back( thread2 );
		addTab( tmp, QString( "Digitizer %1" ).arg( i ) );
	}
}

void OSAPWindow::testing( QString s, QVariant v ) {
	qDebug() << "OSAPWindow::signal( " << s << ", " << v << " )";
}

